package com.common.integrate.video

import android.text.TextUtils
import com.ApiService
import com.alibaba.sdk.android.vod.upload.VODUploadCallback
import com.alibaba.sdk.android.vod.upload.VODUploadClientImpl
import com.alibaba.sdk.android.vod.upload.model.UploadFileInfo
import com.alibaba.sdk.android.vod.upload.model.VodInfo
import com.alibaba.sdk.android.vod.upload.model.VodUploadResult
import com.common.bean.VideoRes
import com.common.bean.json.UploadVideoParams
import com.common.inter.IUploadVideoListener
import com.lagou.baselib.BaseActivity
import com.lagou.baselib.net.httpnetback.BaseCallback
import com.lagou.commonlibs.engine.ThreadUtil
import com.lagou.socketchannel.http.HttpRequest
import com.lagou.socketchannel.http.HttpResponse

/**
 * @date： Crate by wangjunwei on 2022/7/29
 * @description：阿里云文件上传管理类：目前主要用于视频上传
 *
 * 支持两种上传方式：1 部署授权服务获取上传地址和凭证的操作指引请参见使用上传地址和凭证方式上传。
 *               2 部署STS服务获取STS临时Token的操作指引请参见使用STS方式上传。
 *
 **/
class AliyunUploadManager {

    companion object {

        /***
         * 上传类别，职位视频
         */
        private val POSITION_VIDEO = 18

        private var uploader: VODUploadClientImpl? = null

        /***
         * 记录执行压缩的回调序列，方便管理：取消等
         */
        private val uploadVideoMap = mutableMapOf<Int, IUploadVideoListener>()

        /***
         * 当前正在执行的回调tag
         */
        private var currentUploadVideoListenerTag: Int? = null

        /**
         *  上传第 1 步：获取上传地址和凭证
         */
        fun uploadVideoFirst(activity: BaseActivity, videoRes: VideoRes, iUploadVideoListener: IUploadVideoListener, uploadVideoListenerTag: Int){
            if (activity.isActivityDestroyed){
                return
            }

            //只保留一次回调记录，开启新的回调，取消上一次的回调
            currentUploadVideoListenerTag?.let {
                val iUploadVideoListener = uploadVideoMap[it]
                iUploadVideoListener?.onCancelUpload()
                uploadVideoMap.remove(it)
            }
            currentUploadVideoListenerTag = uploadVideoListenerTag
            uploadVideoMap[uploadVideoListenerTag] = iUploadVideoListener

            ApiService.api.getUploadVideoParams(videoRes.name).enqueue(object : BaseCallback<UploadVideoParams>(){
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<UploadVideoParams>?) {
                    if (activity.isActivityDestroyed){
                        iUploadVideoListener.onUploadFailed(null, "","")
                        return
                    }
                    uploadVideo(activity, response?.content, videoRes, uploadVideoListenerTag)
                }
                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<UploadVideoParams>?) {
                    super.onFailure(request, response)
                    iUploadVideoListener.onUploadFailed(null, "","")
                }
            })
        }

        /**
         * 上传视频第二步：阿里云上传视频入口
         * @param activity
         * @param uploadVideoParams 上传需要的参数
         *        uploadAddress：上传地址 uploadAuth：上传凭证等
         * @param videoRes 本地视频信息
         */
        fun uploadVideo(activity: BaseActivity, uploadVideoParams: UploadVideoParams?,
                        videoRes: VideoRes, uploadVideoListenerTag: Int){

            //已经被取消回调，直接返回
            uploadVideoMap[uploadVideoListenerTag] ?: return

            if (uploader == null){
                uploader = VODUploadClientImpl(activity.applicationContext)
            }
            uploader?.setReportEnabled(false)
            uploader?.init(object : VODUploadCallback() {
                override fun onUploadStarted(uploadFileInfo: UploadFileInfo?) {
                    super.onUploadStarted(uploadFileInfo)

                    //空值直接中断流上传流程
                    if (TextUtils.isEmpty( uploadVideoParams?.uploadAuth)
                        || TextUtils.isEmpty( uploadVideoParams?.uploadAddress)){
                        return
                    }
                    //启动上传之后，会触发onUploadStarted回调，通过上传地址和凭证方式上传，需要在该回调方法中设置上传地址和凭证
                    uploader?.setUploadAuthAndAddress(uploadFileInfo,
                        uploadVideoParams?.uploadAuth, uploadVideoParams?.uploadAddress)
                }

                override fun onUploadProgress(
                    info: UploadFileInfo?,
                    uploadedSize: Long,
                    totalSize: Long) {
                    super.onUploadProgress(info, uploadedSize, totalSize)
                    val iUploadVideoProgressListener = uploadVideoMap[uploadVideoListenerTag] ?: return

                    ThreadUtil.ui {
                        iUploadVideoProgressListener?.onUploadProgress(info, uploadedSize, totalSize)
                    }
                }

                override fun onUploadSucceed(info: UploadFileInfo?, result: VodUploadResult?) {
                    super.onUploadSucceed(info, result)
                    val iUploadVideoSuccessListener = uploadVideoMap[uploadVideoListenerTag] ?: return
                    uploadVideoMap.remove(uploadVideoListenerTag)

                    ThreadUtil.ui {
                        iUploadVideoSuccessListener?.onUploadSucceed(info, uploadVideoParams, videoRes)
                    }
                }

                override fun onUploadFailed(
                    info: UploadFileInfo?,
                    code: String?,
                    message: String?) {
                    super.onUploadFailed(info, code, message)
                    val iUploadVideoFailListener = uploadVideoMap[uploadVideoListenerTag] ?: return
                    uploadVideoMap.remove(uploadVideoListenerTag)

                    ThreadUtil.ui {
                        iUploadVideoFailListener?.onUploadFailed(info, code, message)
                    }
                }

                override fun onUploadTokenExpired() {
                    super.onUploadTokenExpired()
                    val iUploadVideoExpiredListener = uploadVideoMap[uploadVideoListenerTag] ?: return

                    //需要调用 uploader?.resumeWithAuth()，传入上传凭证继续上传
                    flushUploadVideoParams(activity, uploadVideoParams?.videoId, uploader, iUploadVideoExpiredListener)
                }
            })

            //构造上需要的参数
            val filePath = videoRes?.destPath
            val vodInfo = VodInfo()
            vodInfo.title = videoRes.name
            vodInfo.cateId = POSITION_VIDEO
            vodInfo.tags = ArrayList<String>()
            vodInfo.tags.add(POSITION_VIDEO.toString())

            uploader?.setPartSize(1024 * 1024)
            uploader?.addFile(filePath, vodInfo)
            uploader?.start()
        }

        /**
         * 取消视频上传
         */
        fun cancelUploadProcess(){
            try {
                currentUploadVideoListenerTag?.let {
                    val iUploadVideoListener = uploadVideoMap[it]
                    iUploadVideoListener?.onCancelUpload()
                    uploadVideoMap.remove(it)
                }
                uploader?.clearFiles()
            }catch (e: Exception){
                e.printStackTrace()
            }
        }


        /***
         * 凭证和地址过期
         * 刷新上传地址和凭证，重新恢复上传
         */
        private fun flushUploadVideoParams(activity: BaseActivity, videoId: String?, uploader: VODUploadClientImpl?, iUploadVideoListener: IUploadVideoListener) {
            if (activity.isActivityDestroyed){
                iUploadVideoListener.onUploadFailed(null, "", "")
                return
            }
            ApiService.api.flushUploadVideoParams(videoId).enqueue(object : BaseCallback<UploadVideoParams>(){
                override fun onSuccess(
                    request: HttpRequest<*>?,
                    response: HttpResponse<UploadVideoParams>?) {
                    if (activity.isActivityDestroyed){
                        iUploadVideoListener.onUploadFailed(null, "", "")
                        return
                    }
                    response?.let {
                        uploader?.resumeWithAuth(it.content?.uploadAuth)
                        return
                    }

                    iUploadVideoListener.onUploadFailed(null, "", "")
                }

                override fun onFailure(
                    request: HttpRequest<*>?,
                    response: HttpResponse<UploadVideoParams>?) {
                    //super.onFailure(request, response)
                    iUploadVideoListener.onUploadFailed(null, "", "")
                }
            })
        }
    }
}