package com.common.widget

import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.view.View
import com.alpha.lagouapk.R
import kotlin.math.asin
import kotlin.math.pow
import kotlin.math.sqrt

/**
 * 三段进度条
 *
 * @author yph
 * @date 2020/6/8
 */
class TripleProgressView(context: Context, attrs: AttributeSet? = null) : View(context, attrs) {
    var bgColor: Int = Color.TRANSPARENT
        set(value) {
            field = value
            invalidate()
        }

    var foregroundColor: Int = Color.TRANSPARENT
        set(value) {
            field = value
            invalidate()
        }

    var radius: Int = 0
        set(value) {
            field = value
            invalidate()
        }

    var gapSize: Int = 0
        set(value) {
            field = value
            invalidate()
        }

    var progress: Float = 0.0f
        set(value) {
            field = value
            invalidate()
        }

    private var tpvWidth: Float = 0.0f
    private var tpvHeight: Float = 0.0f
    private var partWidth: Float = 0.0f

    private val paint: Paint
    private val path: Path
    private val rectF: RectF

    init {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.TripleProgressView)
        foregroundColor = ta.getColor(R.styleable.TripleProgressView_tpv_foreground_color, Color.TRANSPARENT)
        bgColor = ta.getColor(R.styleable.TripleProgressView_tpv_bg_color, Color.TRANSPARENT)
        radius = ta.getDimensionPixelSize(R.styleable.TripleProgressView_tpv_corner_radius, 0);
        gapSize = ta.getDimensionPixelSize(R.styleable.TripleProgressView_tpv_gap_size, 0)
        progress = ta.getFloat(R.styleable.TripleProgressView_tpv_progress, 0.0f)
        ta.recycle()

        paint = Paint()
        paint.isAntiAlias = true
        paint.style = Paint.Style.FILL
        path = Path()
        rectF = RectF()
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        tpvWidth = w.toFloat()
        tpvHeight = h.toFloat()
        partWidth = (w - 2 * gapSize) / 3.0f
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        drawFirst(canvas)
        drawSecond(canvas)
        drawThird(canvas)
    }

    private fun drawFirst(canvas: Canvas?) {
        if (progress <= 1.0f / 3) {
            val pro = (tpvWidth - 2 * gapSize) * progress
            if (pro < radius) {
                path.reset()
                val temp = sqrt(radius.toDouble().pow(2.0) - (radius - pro).toDouble().pow(2.0)).toFloat()
                val j = Math.toDegrees(asin((temp / radius).toDouble())).toFloat()
                path.moveTo(pro, radius + temp)
                rectF.set(0.0f, 0.0f, 2.0f * radius, 2.0f * radius)
                path.arcTo(rectF, 180 - j, j * 2)
                path.close()
                paint.color = foregroundColor
                canvas?.drawPath(path, paint)

                path.reset()
                path.moveTo(pro, radius - temp)
                path.arcTo(rectF, 180 + j, 90 - j)
                path.lineTo(partWidth, 0.0f)
                path.lineTo(partWidth, tpvHeight)
                path.lineTo(radius.toFloat(), tpvHeight)
                path.arcTo(rectF, 90.0f, 90 - j)
                path.close()
                paint.color = bgColor
                canvas?.drawPath(path, paint)
            } else {
                path.reset()
                path.moveTo(pro, tpvHeight)
                path.lineTo(radius.toFloat(), tpvHeight)
                rectF.set(0.0f, 0.0f, 2.0f * radius, 2.0f * radius)
                path.arcTo(rectF, 90.0f, 180.0f)
                path.lineTo(pro, 0.0f)
                path.close()
                paint.color = foregroundColor
                canvas?.drawPath(path, paint)

                rectF.set(pro, 0.0f, partWidth, tpvHeight)
                paint.color = bgColor
                canvas?.drawRect(rectF, paint)
            }
        } else {
            path.reset()
            path.moveTo(radius.toFloat(), 0.0f)
            path.lineTo(partWidth, 0.0f)
            path.lineTo(partWidth, tpvHeight)
            path.lineTo(radius.toFloat(), tpvHeight)
            rectF.set(0.0f, 0.0f, 2.0f * radius, 2.0f * radius)
            path.arcTo(rectF, 90.0f, 180.0f)
            path.lineTo(radius.toFloat(), 0.0f)
            paint.color = foregroundColor
            canvas?.drawPath(path, paint)
        }
    }

    private fun drawSecond(canvas: Canvas?) {
        when {
            progress <= 1.0f / 3 -> {
                rectF.set(partWidth + gapSize, 0.0f, partWidth * 2 + gapSize, tpvHeight)
                paint.color = bgColor
                canvas?.drawRect(rectF, paint)
            }
            progress <= 2.0f / 3 -> {
                val pro = (tpvWidth - 2 * gapSize) * progress
                rectF.set(partWidth + gapSize, 0.0f, pro + gapSize, tpvHeight)
                paint.color = foregroundColor
                canvas?.drawRect(rectF, paint)
                rectF.set(pro + gapSize, 0.0f, partWidth * 2 + gapSize, tpvHeight)
                paint.color = bgColor
                canvas?.drawRect(rectF, paint)
            }
            else -> {
                rectF.set(partWidth + gapSize, 0.0f, partWidth * 2 + gapSize, tpvHeight)
                paint.color = foregroundColor
                canvas?.drawRect(rectF, paint)
            }
        }
    }

    private fun drawThird(canvas: Canvas?) {
        if (progress > 2.0f / 3) {
            val pro = (tpvWidth - 2 * gapSize) * progress
            if (pro + gapSize * 2 >= tpvWidth - radius) {
                path.reset()
                val temp = sqrt(radius.toDouble().pow(2.0) - (radius - (tpvWidth - pro - gapSize * 2)).toDouble().pow(2.0)).toFloat()
                val j = Math.toDegrees(asin((temp / radius).toDouble())).toFloat()
                path.moveTo(pro + gapSize * 2, radius - temp)
                rectF.set((tpvWidth - 2 * radius), 0.0f, tpvWidth, tpvHeight)
                path.arcTo(rectF, -j, j * 2)
                path.close()
                paint.color = bgColor
                canvas?.drawPath(path, paint)

                path.reset()
                path.moveTo(pro + gapSize * 2, radius + temp)
                path.arcTo(rectF, j, 90 - j)
                path.lineTo((partWidth + gapSize) * 2, tpvHeight)
                path.lineTo((partWidth + gapSize) * 2, 0.0f)
                path.lineTo(tpvWidth - radius, 0.0f)
                path.arcTo(rectF, -90.0f, 90 - j)
                path.close()
                paint.color = foregroundColor
                canvas?.drawPath(path, paint)
            } else {
                path.reset()
                path.moveTo(pro + gapSize * 2, 0.0f)
                path.lineTo(tpvWidth - radius, 0.0f)
                rectF.set((tpvWidth - 2 * radius), 0.0f, tpvWidth, tpvHeight)
                path.arcTo(rectF, -90.0f, 180.0f)
                path.lineTo(pro + gapSize * 2, tpvHeight)
                path.close()
                paint.color = bgColor
                canvas?.drawPath(path, paint)

                rectF.set((partWidth + gapSize) * 2, 0.0f, pro + gapSize * 2, tpvHeight)
                paint.color = foregroundColor
                canvas?.drawRect(rectF, paint)
            }
        } else {
            path.reset()
            path.moveTo(tpvWidth - partWidth, 0.0f)
            path.lineTo((tpvWidth - radius), 0.0f)
            rectF.set((tpvWidth - 2 * radius), 0.0f, tpvWidth, tpvHeight)
            path.arcTo(rectF, -90.0f, 180.0f)
            path.lineTo(tpvWidth - partWidth, tpvHeight)
            path.close()
            paint.color = bgColor
            canvas?.drawPath(path, paint)
        }
    }

}